#############################################################################
#
# Copyright (c) 2006 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Bootstrap a buildout-based project

Simply run this script in a directory containing a buildout.cfg.
The script accepts buildout command-line options, so you can
use the -c option to specify an alternate configuration file.

$Id$
"""
import glob
import os
import shutil
import sys
import tempfile
import urllib2

EZ_SETUP = 'http://peak.telecommunity.com/dist/ez_setup.py'
eggs_folder = os.path.join(os.path.abspath(os.curdir), 'eggs')
if not os.path.exists(eggs_folder):
    os.mkdir(eggs_folder)
sys.path.insert(0, eggs_folder)

downloads_folder = os.path.join(os.path.abspath(os.curdir), 'downloads')
if not os.path.exists(downloads_folder):
    os.mkdir(downloads_folder)
sys.path.insert(0, downloads_folder)

def _import(egg_name):
    mod = __import__(egg_name)
    egg_dir = os.path.dirname(mod.__file__)
    egg_dir = os.path.split(egg_dir)[0]
    egg_location = os.path.split(egg_dir)[0]
    if not egg_location.startswith(eggs_folder):
        print 'Copying %s into the buildout' % egg_name
        path_ = _extract_path(egg_name, egg_location)
        if len(path_) > 0:
            path_ = path_[0]
            folder_name = os.path.split(path_)[-1]
            target = os.path.join(eggs_folder, folder_name)
            if target.lower() == path_.lower():
                return 
            if os.path.exists(target):
                shutil.rmtree(target)
            shutil.copytree(path_, target)

def _extract_path(egg_name, folder=eggs_folder):
    egg = glob.glob(os.path.join(folder, '%s-*' % egg_name))
    egg.sort()
    egg.reverse()
    return egg

def link_egg(working_set, egg_name, required=True):
    egg = _extract_path(egg_name)
    if egg != []:
        egg_location = egg[0]
        print 'Using %s from %s' % (egg_name, egg_location)
        working_set.add_entry(egg_name)
        working_set.require(egg_name)
    else:
        try:
            _import(egg_name)
        except ImportError:
            # calling easy_install
            setuptools_parse = pkg_resources.Requirement.parse('setuptools')
            locations = '%s;%s' % (setuptools_parse, eggs_folder)
            PATHS = dict(os.environ, PYTHONPATH=locations)
            cmd = 'from setuptools.command.easy_install import main; main()'
            if sys.platform == 'win32':
                cmd = '"%s"' % cmd # work around spawn lamosity on windows

            print os.system("%s -c '%s' -mqNxd %s %s" % (sys.executable, cmd, eggs_folder, egg_name))

            # retrying import
            working_set.add_entry(egg_name)
            working_set.require(egg_name)
            _import(egg_name)

    #sys.path.insert(0, egg_location)
    #working_set.add_entry(egg_name)
    #working_set.require(egg_name)
    
    #return egg_location 

#
# setuptool install
#

# let's try to see if the setuptools is not in the eggs folder already
setuptools = glob.glob(os.path.join(eggs_folder, 'setuptools-*'))
setuptools.sort()
setuptools.reverse()
if setuptools:
    print 'Using setuptools from %s' % setuptools[0]
    sys.path.insert(0, setuptools[0])
    import pkg_resources
else:
    # next, try to see if the environment has it
    try:
        import pkg_resources
        print 'Getting setuptools from %s' % \
            os.path.dirname(pkg_resources.__file__)
    except ImportError:
        print 'Installing setuptools in eggs folder'
        # last option, let's try to get it
        ez = {}
        exec urllib2.urlopen(EZ_SETUP).read() in ez
        ez['use_setuptools'](to_dir=eggs_folder, download_delay=0)
        
ws = pkg_resources.working_set
link_egg(ws, 'zc.buildout')
link_egg(ws, 'zope.testing', required=False)

ws.add_entry(eggs_folder)

import zope.testing
from zc.buildout.buildout import main

main(sys.argv[1:] + ['bootstrap'])




# -*- coding: utf-8 -*-
# Copyright (C)2007 ${str($author) or $empty}

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Generic Test case for ${str($project)} doctest
"""
__docformat__ = 'restructuredtext'

import unittest
import doctest
import sys
import os
import shutil
import popen2

from zope.testing import doctest

current_dir = os.path.abspath(os.path.dirname(__file__))

def rmdir(*args):
    dirname = os.path.join(*args)
    if os.path.isdir(dirname):
        shutil.rmtree(dirname)

def sh(cmd):
    """XXX this is intrusive
    -> need to be root to install python packages
    in some cases
    """
    _cmd = cmd
    print cmd
    if _cmd.startswith('python') or \
       _cmd.startswith('paster'):
        _cmd = execdir + os.path.sep + cmd
    # launch command 2 times to see what append and be able to test in doc tests
    os.system(_cmd)
    child_stdout_and_stderr, child_stdin = popen2.popen4(_cmd)
    child_stdin.close()
    print child_stdout_and_stderr.read()

def ls(*args):
    dirname = os.path.join(*args)
    if os.path.isdir(dirname):
        filenames = os.listdir(dirname)
        for filename in sorted(filenames):
            if os.path.isdir(os.path.join(dirname, filename)):
                print 'd %s' % filename
            else:
                print '- %s' % filename
    else:
        print 'No directory named %s' % dirname

def cd(*args):
    dirname = os.path.join(*args)
    os.chdir(dirname)


def config(filename):
    return os.path.join(current_dir, filename)

def cat(*args):
    filename = os.path.join(*args)
    if os.path.isfile(filename):
        print open(filename).read()
    else:
        print 'No file named %s' % filename

def touch(*args, **kwargs):
    filename = os.path.join(*args)
    open(filename, 'w').write(kwargs.get('data',''))

execdir = os.path.abspath(os.path.dirname(sys.executable))
tempdir = os.getenv('TEMP','/tmp')

def doc_suite(test_dir, setUp=None, tearDown=None, globs=None):
    """Returns a test suite, based on doctests found in /doctest."""
    suite = []
    if globs is None:
        globs = globals()

    flags = (doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE |
             doctest.REPORT_ONLY_FIRST_FAILURE)

    package_dir = os.path.split(test_dir)[0]
    if package_dir not in sys.path:
        sys.path.append(package_dir)

    doctest_dir = os.path.join(package_dir, 'docs')

    # filtering files on extension
    docs = [os.path.join(doctest_dir, doc) for doc in
            os.listdir(doctest_dir) if doc.endswith('.txt')]

    for test in docs:
        suite.append(doctest.DocFileSuite(test, optionflags=flags,
                                          globs=globs, setUp=setUp,
                                          tearDown=tearDown,
                                          module_relative=False))

    return unittest.TestSuite(suite)

def test_suite():
    """returns the test suite"""
    return doc_suite(current_dir)

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')




# -*- coding: utf-8 -*-
## Copyright (C)2007 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
""" Template definitions
"""
import os
import shutil
import copy

from paste.script import templates

from zopeskel.plone import Plone
from zopeskel.basic_namespace import BasicNamespace
from zopeskel.nested_namespace import NestedNamespace
from zopeskel.base import var
from zopeskel.base import get_var

from addon import AddOn

class IWPloneProject(templates.Template):
    """ Generates a Plone 3 project tree with Ingeniweb packages
    """
    _template_dir = 'templates/iw_plone_project'
    summary = 'A Plone 3 project tree for Ingeniweb projects'
    use_cheetah = True

    vars = [var('project_name', 'Project Name'),
            var('project_repo', 'Project Subversion root'),
            var('zope_user', 'Zope root admin user', default='admin'),
            var('zope_password', 'Zope root admin password'),
            var('http_port', 'HTTP port', default=8080),
            var('debug_mode', 'Should debug mode be "on" or "off"?', default='off'),
            var('verbose_security', 'Should verbose security be "on" or "off"?', default='off')]

class IWPython(BasicNamespace):
    """ Generates a pure Python lib for 'iw' namespace
    """
    _template_dir = 'templates/iw_python'
    summary = "A project with a namespace package"
    required_templates = ['basic_namespace']

    vars = [
        var('namespace_package', 'Namespace package', default='iw'),
        var('package', ('The package contained namespace package '
                        '(like example)'), default='example'),
        var('version', 'Version', default='0.1'),
        var('description', 'One-line description of the package'),
        var('author', 'Author name', default='Ingeniweb'),
        var('author_email', 'Author email',
            default='support@ingeniweb.com'),
        var('keywords', 'Space-separated keywords/tags'),
        var('url', 'URL of homepage'),
        var('license_name', 'License name', default='GPL'),
        var('zip_safe', 'True/False: if the package can be distributed '
            'as a .zip file', default=False),
        ]

class IWContent(AddOn):

    _template_dir = 'templates/iw_content'
    summary = "A content type"

    vars = [
        var('author', 'Author name', default='Ingeniweb'),
        var('author_email', 'Author email',
            default='support@ingeniweb.com'),
        ]

    def pre(self, command, output_dir, vars):
        if not os.path.isdir('tests'):
            raise RuntimeError('You cant create a content without a tests dir')
        if not os.path.isdir('doctests'):
            raise RuntimeError('You cant create a content without a doctests dir')
        vars['type_name'] = vars['project'].lower()
        vars['class_name'] = vars['project']
        projectname = self.getProjectName()
        if '.' in projectname:
            # product is a namespace
            vars['namespace'] = projectname
            product_name = [n.title() for n in projectname.split('.')]
            product_name = ''.join(product_name)
            vars['product_name'] = product_name
        else:
            # product is a standard zope Product
            vars['namespace'] = 'Products.%s' % projectname
            vars['product_name'] = projectname

class IWPlone(Plone):
    """A plone 3 namespaced product"""
    use_cheetah = True
    vars = copy.deepcopy(Plone.vars)
    _template_dir = 'templates/iw_plone'
    summary = "A Plone 3 namespaced product"
    
    get_var(vars, 'namespace_package').default = 'iw'
    get_var(vars, 'author').default = 'Ingeniweb'
    get_var(vars, 'author_email').default = 'support@ingeniweb.com'
    get_var(vars, 'url').default = ('https://ingeniweb.svn.sourceforge.net/'
                                    'svnroot/ingeniweb/iw.example')




# -*- coding: utf-8 -*-
## Copyright (C)2007 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
""" Add on tests
"""
import unittest
import os
import shutil

from ingeniskel.addon import AddOn

curdir = os.path.dirname(__file__)

class AddOnTest(unittest.TestCase):

    def test_diff_1(self):
        #    
        # case 1 normal injection (no file)
        #
        adder = AddOn('xx')

        path = os.path.join(curdir, 'src')
        dirname = os.path.join(curdir, 'dest')
        filename = 'configure.zcml'
        waited = os.path.join(curdir, 'waited', filename)
        result = os.path.join(curdir, 'dest', filename)
        src = os.path.join(path, filename) 
        self.assert_(not os.path.exists(result))
        try:
            # first injection
            adder._check_file(filename, path, dirname)
            self.assert_(os.path.exists(result))
            
            # second injection 
            adder._check_file(filename, path, dirname)

            # checking the result
            result = os.path.join(curdir, 'dest', filename)
            self.assertEquals(open(result).read(), open(src).read())  
        finally:
            # removing dest
            os.remove(result)

    def test_diff_2(self):
        """testing file injection"""
        #    
        # case 2 diff injection (no exists)
        #
        adder = AddOn('xx')

        path = os.path.join(curdir, 'src')
        dirname = os.path.join(curdir, 'dest')
        filename = 'configure2.zcml'
        waited = os.path.join(curdir, 'waited', filename)
        result = os.path.join(curdir, 'dest', filename)

        # copying result before change
        shutil.copyfile(result, os.path.join(dirname, 'old'))

        try:
            # first injection, this should merge
            adder._check_file(filename, path, dirname)
            self.assert_(os.path.exists(result))
            
            # second injection, this should not merge anymore
            adder._check_file(filename, path, dirname)

            # checking the result
            result = os.path.join(curdir, 'dest', filename)
            waited = os.path.join(curdir, 'waited', filename)

            self.assertEquals(open(result).read(), open(waited).read())  
        
        # cleaning
        finally:
            os.remove(result)
            shutil.move(os.path.join(dirname, 'old'), result)

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(AddOnTest))
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')




# -*- coding: utf-8 -*-
## Copyright (C)2007 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Setup definitions
"""
import ez_setup
ez_setup.use_setuptools()

from setuptools import setup, find_packages
import sys, os

version = '0.2.6'

setup(name='IngeniSkel',
      version=version,
      description="A collection of skeletons for quickstarting projects with Ingeniweb products.",
      long_description="""
A collection of skeletons for quickstarting Ingeniweb Zope projects.
""",
      classifiers=[
        "Development Status :: 3 - Alpha",
        "Framework :: Zope2",
        "Framework :: Zope3",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='web zope command-line skeleton project',
      author='Ingeniweb',
      author_email='support@ingeniweb.com',
      url='http://ingeniweb.svn.sourceforge.net/svnroot/ingeniweb/IngeniSkel',
      packages=find_packages(exclude=['ez_setup']),
      include_package_data=True,
      zip_safe=False,
      test_suite = "tests.test_ingeniskeldocs.test_suite",
      install_requires=[
        "ZopeSkel>=1.5.1"
      ],
      entry_points="""
      [paste.paster_create_template]
      iw_plone_project = ingeniskel:IWPloneProject
      iw_plone = ingeniskel:IWPlone
      iw_python = ingeniskel:IWPython
      iw_content = ingeniskel:IWContent
      """,
      )




# -*- coding: utf-8 -*-
## Copyright (C)2007 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
""" Add on
"""
import os
import shutil
import re
import difflib

from zopeskel.basic_namespace import BasicNamespace

class AddOn(BasicNamespace):
    required_templates = []
    
    def getProjectName(self):
        if not os.path.isfile('config.py'):
            raise RuntimeError('You cant create a content without a config.py')
        projectname = None
        fd = open('config.py')
        for line in fd:
            if line.startswith('PROJECTNAME'):
                projectname = line.split('=')[1]
                projectname = projectname.strip()
                projectname = projectname.replace('"','')
                projectname = projectname.replace("'",'')
        fd.close()
        if not projectname:
            raise RuntimeError("Can't retrieve project name in config.py")
        return projectname

    def _diff(self, target, addon):
        """marge addon into src"""
        # diff that sees if the file is a tagged file
        # if so, inject the content just before the end bracket
        # otherwised, appends
        if addon in target:
            return target
        # let's find the diff, then merge it
        def _merge(line):
            if line[:2] in ('+ ', '- ', '  '):
                return line[2:]
            return ''
        delta = [_merge(el) for el in 
                 difflib.ndiff(target.splitlines(1), 
                               addon.splitlines(1))]
        # let's inject it
        return ''.join(delta)
                
    def _check_file(self, filename, path, dirname):
        """work on a file"""   
        src = os.path.join(path, filename)
        if os.path.isdir(src):
            # let's recurse
            for element in os.listdir(src):
                dir_ = os.path.split(src)[-1]
                subdirname = os.path.join(dirname, dir_)
                if not os.path.exists(subdirname):
                    os.mkdir(subdirname)
                self._check_file(element, src, subdirname)
            return

        dst = os.path.join(dirname, filename)
        if not os.path.exists(dst):
            shutil.copyfile(src, dst)
            if os.path.isdir(os.path.join(dirname, '.svn')):
                os.popen('svn add %s' % dst)
        else:
            dst_content = open(dst).read()
            src_content = open(src).read()
            if dst_content == src_content: 
                print '    Skipping %s. file exist' % dst
            else:
                print '    Making diff for %s' % dst
                final_content = self._diff(dst_content, src_content)
                f = open(dst, 'w')
                try:
                    f.write(final_content)
                finally:
                    f.close() 

    def post(self, command, output_dir, vars):
        """post-templating"""
        basedir = vars['project']
        print 'Moving package %s to product %s' % (vars['project'], 
                                                   vars['product_name'])
        
        package_dir = '.'
        for element in os.listdir(basedir):
            path = os.path.join(basedir, element)
            print '  Copying %s to %s' % (path, package_dir)
            if not os.path.isdir(path):
                # single file
                self._check_file(element, basedir, package_dir)
            else:
                # subfolder
                extracted_folder = os.path.split(path)[-1]
                subfolder = os.path.join(package_dir, extracted_folder)
                if not os.path.exists(subfolder):
                    os.mkdir(subfolder)
                if os.path.exists(os.path.join(package_dir, '.svn')):
                    os.popen('svn add %s' % subfolder)
                # copying subfolder elements
                for filename in os.listdir(path):
                    self._check_file(filename, path, subfolder)

        print 'Removing %s' % basedir
        shutil.rmtree(basedir)

